import strutils, sequtils

#? validate if user input is of correct type
proc getInt: int =
    while(true):
        try:
            return parseInt(readline(stdin))
        except:
            echo "Please enter an integer, try again."

#? validate if user input is of correct type
proc getFloat: float =
    while(true):
        try:
            return parseFloat(readline(stdin))
        except:
            echo "Please enter a number, try again."

#? prints a matrix to the standard output
proc printMatrix*(matrix: seq[seq[float]]) =
    let row: int = matrix.len
    for i in countup(1, row):
        echo matrix[i-1]

#? get elements of matrix from standard input
proc fillMatrix*(matrix: var seq[seq[float]], row, col: int) =
    for i in countup(1, row):
        for j in countup(1, col):
            echo "Enter item:"
            matrix[i-1].add(getFloat())
            matrix[i-1].delete(0)
        echo matrix[i-1]

#? calculate dot product
proc calcDot(matrix1: seq[seq[float]], matrix2: seq[seq[float]]): seq[seq[float]] =
    let col1: int = matrix1[0].len
    let row1: int = matrix1.len
    let col2: int = matrix2[0].len
    let row2: int = matrix2.len
    var col, row: int

    if col1 == row2:
        col = col2
        row = row1
    else:
        quit "Matrix dimensions mismatched, operation invalid!", QuitFailure

    var matrix = newSeqWith(row, newSeq[float](col))

    for i in countup(0, row1-1):
        for j in countup(0, col2-1):
            for k in countup(0, col1-1):
                matrix[i][j] = matrix[i][j] + matrix1[i][k] * matrix2[k][j]
    return matrix

#? calculate cross product
proc calcCross(vector1: array[3, float], vector2: array[3, float]): array[3, float] =
    let i: float = vector1[1] * vector2[2] - vector1[2] * vector2[1] 
    let j: float = vector1[2] * vector2[0] - vector1[0] * vector2[2]
    let k: float = vector1[0] * vector2[1] - vector1[1] * vector2[0]
    result = [i, j, k]

proc dot* =
    echo "MATRIX DOT PRODUCT"

    #? record first matrix
    echo "Enter number of rows in the first matrix:"
    let r1: int = getInt() 
    echo "Enter number of columns in the first matrix:"
    let c1: int = getInt()
    var m1 = newSeqWith(r1, newSeq[float](c1))
    procedures.fillMatrix(m1, r1, c1)

    echo ""

    #? record second matrix
    echo "Enter number of rows in the second matrix:"
    let r2: int = getInt()
    echo "Enter number of columns in the second matrix:"
    let c2: int = getInt()
    var m2 = newSeqWith(r2, newSeq[float](c2))
    procedures.fillMatrix(m2, r2, c2)

    #? resultent matrix
    var m: seq[seq[float]]
    m = calcDot(m1, m2)

    echo "\nFirst matrix is:"
    printMatrix(m1)
    echo "\nSecond matrix is:"
    printMatrix(m2)  
    echo "\nResult matrix is:"
    printMatrix(m)

proc cross* =
    echo "VECTOR CROSS PRODUCT"
    
    type
        VECTOR = array[3, float]
    var 
        v1: VECTOR
        v2: VECTOR

    echo "Enter numbers in the first vector:"
    for i in 0..2:
        echo "Enter item:"
        v1[i] = getFloat()

    echo "Enter numbers in the second vector:"
    for i in 0..2:
        echo "Enter item:"
        v2[i] = getFloat()

    #? resultent vector
    let v = calcCross(v1, v2)

    echo v1, " \u2A2F ", v2
    echo "\nResult vector is:"
    echo v
